/*
 * test-trid-lu.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "trid-lu.h"

double MAXINT;

void multiply(int n, double *Ax, double *AL, double *AD, double *AU, double *x)
{
   int i, nm1;
   nm1 = n - 1;
   Ax[0] = AD[0] * x[0] + AU[0] * x[1];
   for (i = 1; i < nm1; i++)
     Ax[i] = AL[i] * x[i-1] + AD[i] * x[i] + AU[i] * x[i+1];
   Ax[nm1] = AL[nm1] * x[nm1-1] + AD[nm1] * x[nm1];
}

double drandom()
{
  return random() / MAXINT;
}

int main()
{
  int i, n;
  double *al, *ad, *au, *b, *x, error;

  MAXINT = pow(2.0, 31.0) - 1.0;

#ifdef HEAT
  n = 7;
#else
  n = 20;
#endif
  al = (double *)malloc(sizeof(double) * n);
  ad = (double *)malloc(sizeof(double) * n);
  au = (double *)malloc(sizeof(double) * n);
  b  = (double *)malloc(sizeof(double) * n);
  x  = (double *)malloc(sizeof(double) * n);
  for (i = 0; i < n; i++) {
#ifdef HEAT
    al[i] = - 0.5; ad[i] = 2.0; au[i] = - 0.5;
#else
    al[i] = - drandom(); ad[i] = 1 + 2 * drandom(); au[i] = - drandom();
#endif
    x[i] = i + 1;
  }
  multiply(n, b, al, ad, au, x);
  trilu(n, al, ad, au);
#ifdef HEAT
  for (i = 0; i < n; i++) {
    if (i == 0)
      printf("                     %20.15g %20.15g\n", ad[i], au[i]);
    else if (i != n-1)
      printf("%20.15g %20.15g %20.15g\n", al[i], ad[i], au[i]);
    else
      printf("%20.15g %20.15g\n", al[i], ad[i]);
  }
#endif
  trisol(n, al, ad, au, b);
  error = 0.0;
  for (i = 0; i < n; i++) {
    error += fabs(x[i] - b[i]);
    printf("x[%d]=%g, b[%d]=%g, diff=%g\n", i, x[i], i, b[i], x[i] - b[i]);
  }
  printf("L1 error=%g\n", error);
  return 0;
}
